"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cross_fetch_1 = require("cross-fetch");
// The target will be window in a browser and global in node
var target = ((typeof window === "undefined") ? global : window);
// Make sure console.debug exists (it doesn't in node)
if (!target.console.debug)
    target.console.debug = target.console.log;
// Save the original console
var originalConsole = target.console;
// Get information about the platform
var platform = require("platform");
function getLevel(level) {
    switch (level.toLowerCase()) {
        case "log": return 7;
        case "debug": return 7;
        case "info": return 6;
        case "notice": return 5;
        case "warn": return 4;
        case "error": return 3;
        case "critical": return 2;
        case "alert": return 1;
        case "emergency": return 0;
    }
}
if (typeof Object.assign != 'function') {
    // Must be writable: true, enumerable: false, configurable: true
    Object.defineProperty(Object, "assign", {
        value: function assign(target, varArgs) {
            'use strict';
            if (target == null) { // TypeError if undefined or null
                throw new TypeError('Cannot convert undefined or null to object');
            }
            var to = Object(target);
            for (var index = 1; index < arguments.length; index++) {
                var nextSource = arguments[index];
                if (nextSource != null) { // Skip over if undefined or null
                    for (var nextKey in nextSource) {
                        // Avoid bugs when hasOwnProperty is shadowed
                        if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
                            to[nextKey] = nextSource[nextKey];
                        }
                    }
                }
            }
            return to;
        },
        writable: true,
        configurable: true
    });
}
function configure(opts) {
    var capitalize = function (s) { return s.length > 0 ? s[0].toUpperCase() + s.substring(1) : ""; };
    function makeLogger(originalFn, level) {
        return function (message) {
            var contexts = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                contexts[_i - 1] = arguments[_i];
            }
            var _a;
            // If the first element of contexts is a string then that is the fullMessage
            var fullMessage = null;
            if (contexts.length > 0 && typeof contexts[0] === "string") {
                fullMessage = contexts.shift();
            }
            // Put all the remaining contexts together into a single object
            var additionalFields = (_a = Object).assign.apply(_a, [{}].concat(contexts));
            // Get the severity - this either comes from the console function, or from severity in the contexts
            var severity = additionalFields.severity !== undefined ? getLevel(additionalFields.severity) : getLevel(level);
            delete additionalFields.severity;
            // Only log to Graylog if the severity is at least opts.level
            if (severity <= getLevel(opts.level || "log")) {
                // If any of the additional fields are called 'id', rename them to 'id_' (since GELF reserves _id)
                if (Object.keys(additionalFields).filter(function (key) { return key === "id"; }).length === 1) {
                    additionalFields["id_"] = additionalFields.id;
                    delete additionalFields.id;
                }
                // Construct the basic gelf message
                var gelfMessage = {
                    "version": "1.1",
                    "host": opts.host,
                    "short_message": message,
                    "level": severity,
                };
                // If message is an Error then extract useful bits
                if (Object.prototype.toString.call(message) === "[object Error]") {
                    gelfMessage.short_message = message.message;
                    gelfMessage.full_message = message.stack;
                }
                // Add the full_message, if there is one
                if (fullMessage)
                    gelfMessage.full_message = fullMessage;
                // Add all the information we got from platform.js
                gelfMessage.platform = { platform: platform };
                // Plus the screen size details, if we can get them
                if (target.screen) {
                    gelfMessage.platform.platform.screen = {
                        availHeight: target.screen.availHeight,
                        availLeft: target.screen.availLeft,
                        availTop: target.screen.availTop,
                        availWidth: target.screen.availWidth,
                        colorDepth: target.screen.colorDepth,
                        height: target.screen.height,
                        pixelDepth: target.screen.pixelDepth,
                        width: target.screen.width
                    };
                    // Explicitly check for orientation as it doesn't necessarily exist on Safari
                    if (target.screen.orientation) {
                        gelfMessage.platform.platform.screen.orientation = {
                            angle: target.screen.orientation.angle,
                            type: target.screen.orientation.type
                        };
                    }
                }
                // Add any static properties passed in at config time
                for (var key in (opts.staticProperties || {}))
                    gelfMessage["_" + key] = opts.staticProperties[key];
                // Add all the optional parameters, with underscores
                for (var key in additionalFields)
                    gelfMessage["_" + key] = additionalFields[key];
                // Create a headers object
                var headers = new cross_fetch_1.Headers();
                headers.append('Content-Type', 'application/json');
                // And finally make the request against the Graylog endpoint.  We don't care if it fails.
                cross_fetch_1.fetch(opts.endpoint, {
                    method: "post",
                    body: JSON.stringify(gelfMessage),
                    headers: headers
                }).catch(function (err) {
                    if (opts.debug)
                        target.console["originalError"](err.toString());
                });
            }
            // Write to the console
            originalFn.call.apply(originalFn, [target.console, message + (fullMessage ? ": " + fullMessage : "")].concat(contexts));
        };
    }
    // Overwrite the console methods with our wrapped versions (renaming the original)
    ["log", "debug", "info", "warn", "error"].forEach(function (level) {
        target.console["original" + capitalize(level)] = target.console[level];
        target.console[level] = makeLogger(originalConsole[level], level);
    });
}
exports.configure = configure;
//# sourceMappingURL=index.js.map